class OrderInfo:
    order_id = None
    order_price = None
    order_name = None
    payment_method = None
    store = None
    country = None
    image_url = None
    success_url = None

    def __init__(self):
        pass



SETTINGS = {
    "sdk_version": "1.0.0",
    # "api_live_uri": "http://api.compropago.com/v1/",
    # "api_sandbox_uri": "http://api.compropago.com/v1/"

    "api_live_uri": "http://api-staging-compropago.herokuapp.com/v1/",
    "api_sandbox_uri": "http://api-staging-compropago.herokuapp.com/v1/"
}



from compropago.factory.serialize import Serialize
import json


class Factory:
    def __init__(self):
        pass

    @staticmethod
    def get_instance_of(class_name, data=None):
        if data and isinstance(data, str):
            data = json.loads(data)
        elif data and isinstance(data, dict):
            data = data

        if class_name is 'CpOrderInfo':
            return Serialize.cp_order_info(data)
        elif class_name is 'Customer':
            return Serialize.customer(data)
        elif class_name is 'EvalAuthInfo':
            return Serialize.eval_auth_info(data)
        elif class_name is 'FeeDetails':
            return Serialize.fee_details(data)
        elif class_name is 'InstructionDetails':
            return Serialize.instruction_details(data)
        elif class_name is 'Instructions':
            return Serialize.instructions(data)
        elif class_name is 'NewOrderInfo':
            return Serialize.new_order_info(data)
        elif class_name is 'OrderInfo':
            return Serialize.order_info(data)
        elif class_name is 'PlaceOrderInfo':
            return Serialize.place_order_info(data)
        elif class_name is 'Provider':
            return Serialize.provider(data)
        elif class_name is 'ListProviders':
            array_aux = []

            for provider in data:
                array_aux.append(Serialize.provider(provider))

            return array_aux
        elif class_name is 'SmsInfo':
            return Serialize.sms_info(data)
        elif class_name is 'SmsData':
            return Serialize.sms_data(data)
        elif class_name is 'SmsObject':
            return Serialize.sms_object(data)
        elif class_name is 'Webhook':
            return Serialize.webhook(data)
        elif class_name is 'ListWebhooks':
            array_aux = []

            for webhook in data:
                array_aux.append(Serialize.webhook(webhook))

            return array_aux
        else:
            raise Exception('Object not in factory.')



class InstructionDetails:
    amount = None
    store = None
    payment_amount = None
    payment_store = None
    bank_account_holder_name = None
    bank_account_number = None
    bank_reference = None
    company_reference_name = None
    company_reference_number = None
    company_bank_number = None
    order_reference_number = None
    bank_name = None

    def __init__(self):
        pass



from compropago.factory.models.orderinfo import OrderInfo
from compropago.factory.models.feedetails import FeeDetails
from compropago.factory.models.instructions import Instructions


class NewOrderInfo:
    id = None
    short_id = None
    object = None
    status = None
    created = None
    exp_date = None
    live_mode = None
    order_info = None
    fee_details = None
    instructions = None

    def __init__(self):
        self.order_info = OrderInfo()
        self.fee_details = FeeDetails()
        self.instructions = Instructions()



import compropago.config as config
from compropago.factory.models.placeorderinfo import PlaceOrderInfo
from compropago.factory.models.cporderinfo import CpOrderInfo
from compropago.factory.models.customer import Customer
from compropago.factory.models.evalauthinfo import EvalAuthInfo
from compropago.factory.models.feedetails import FeeDetails
from compropago.factory.models.instructiondetails import InstructionDetails
from compropago.factory.models.instructions import Instructions
from compropago.factory.models.neworderinfo import NewOrderInfo
from compropago.factory.models.orderinfo import OrderInfo
from compropago.factory.models.provider import Provider
from compropago.factory.models.smsinfo import SmsInfo
from compropago.factory.models.smsobject import SmsObject
from compropago.factory.models.smsdata import SmsData
from compropago.factory.models.webhook import Webhook


class Serialize:
    def __init__(self):
        pass

    @staticmethod
    def place_order_info(data=None):
        if not data:
            return PlaceOrderInfo(None, None, None, None, None)
        else:
            return PlaceOrderInfo(
                data['order_id'],
                data['order_name'],
                data['order_price'],
                data['customer_name'],
                data['customer_email'],
                data['payment_type'] if 'payment_type' in data else 'OXXO',
                data['currency'] if 'currency' in data else 'MXN',
                data['image_url'] if 'image_url' in data else "",
                data['app_client_name'] if 'app_client_name' in data else "python-sdk",
                data['app_client_version'] if 'app_client_version' in data else config.SETTINGS['sdk_version']
            )

    @staticmethod
    def cp_order_info(data=None):
        if not data:
            return CpOrderInfo()
        else:
            obj = CpOrderInfo()

            obj.id = data['id']
            obj.type = data['type']
            obj.object = data['object']
            obj.created = data['created']
            obj.paid = data['paid']
            obj.amount = data['amount']
            obj.livemode = data['livemode']
            obj.currency = data['currency']
            obj.refunded = data['refunded']
            obj.fee = data['fee']

            obj.fee_details = Serialize.fee_details(data['fee_details'])
            obj.order_info = Serialize.order_info(data['order_info'])
            obj.customer.customer_name = Serialize.customer(data['customer'])

            obj.captured = data['captured']
            obj.failure_message = data['failure_message']
            obj.failure_code = data['failure_code']
            obj.amount_refunded = data['amount_refunded']
            obj.description = data['description']
            obj.dispute = data['dispute']

            return obj

    @staticmethod
    def customer(data=None):
        if not data:
            return Customer()
        else:
            obj = Customer()

            obj.customer_name = data['customer_name']
            obj.customer_email = data['customer_email']
            obj.customer_phone = data['customer_phone']

            return obj

    @staticmethod
    def eval_auth_info(data=None):
        if not data:
            return EvalAuthInfo()
        else:
            obj = EvalAuthInfo()

            obj.type = data['type']
            obj.livemode = data['livemode']
            obj.mode_key = data['mode_key']
            obj.message = data['message']
            obj.code = data['code']

            return obj

    @staticmethod
    def fee_details(data=None):
        if not data:
            return FeeDetails()
        else:
            obj = FeeDetails()

            obj.amount = data['amount'] if 'amount' in data else None
            obj.currency = data['currency'] if 'currency' in data else None
            obj.type = data['type'] if 'type' in data else None
            obj.description = data['description'] if 'description' in data else None
            obj.application = data['application'] if 'application' in data else None
            obj.tax_percent = data['tax_percent'] if 'tax_percent' in data else None
            obj.amount_refunded = data['amount_refunded'] if 'amount_refunded' in data else None
            obj.tax = data['tax'] if 'tax' in data else None

            return obj

    @staticmethod
    def instruction_details(data=None):
        if not data:
            return InstructionDetails()
        else:
            obj = InstructionDetails()

            obj.amount = data['amount'] if 'amount' in data else None
            obj.store = data['store'] if 'store' in data else None
            obj.payment_amount = data['payment_amount'] if 'payment_amount' in data else None
            obj.payment_store = data['payment_store'] if 'payment_store' in data else None
            obj.bank_account_holder_name = data['bank_account_holder_name'] if 'bank_account_holder_name' in data else None
            obj.bank_account_number = data['bank_accont_number'] if 'bank_accont_number' in data else None
            obj.bank_reference = data['bank_reference'] if 'bank_reference' in data else None
            obj.company_reference_name = data['company_reference_name'] if 'company_reference_name' in data else None
            obj.company_reference_number = data['company_reference_number'] if 'company_reference_number' in data else None
            obj.company_bank_number = data['company_bank_number'] if 'company_bank_number' in data else None
            obj.order_reference_number = data['order_reference_number'] if 'order_reference_number' in data else None
            obj.bank_name = data['bank_name'] if 'bank_name' in data else None

            return obj

    @staticmethod
    def instructions(data=None):
        if not data:
            return Instructions()
        else:
            obj = Instructions()

            obj.description = data['description']
            obj.step_1 = data['step_1']
            obj.step_2 = data['step_2']
            obj.step_3 = data['step_3']
            obj.note_extra_comition = data['note_extra_comition']
            obj.note_expiration_date = data['note_expiration_date']
            obj.note_confirmation = data['note_confirmation']

            obj.details = Serialize.instruction_details(data['details'])

            return obj

    @staticmethod
    def new_order_info(data=None):
        if not data:
            return NewOrderInfo()
        else:
            obj = NewOrderInfo()

            obj.id = data['id']
            obj.short_id = data['short_id']
            obj.object = data['object']
            obj.status = data['status']
            obj.created = data['created']
            obj.exp_date = data['exp_date']
            obj.live_mode = data['live_mode']
            obj.order_info = Serialize.order_info(data['order_info'])
            obj.fee_details = Serialize.fee_details(data['fee_details'])
            obj.instructions = Serialize.instructions(data['instructions'])

            return obj

    @staticmethod
    def order_info(data=None):
        if not data:
            return OrderInfo()
        else:
            obj = OrderInfo()

            obj.order_id = data['order_id'] if 'order_id' in data else None
            obj.order_price = data['order_price'] if 'order_price' in data else None
            obj.order_name = data['order_name'] if 'order_name' in data else None
            obj.payment_method = data['payment_method'] if 'payment_method' in data else None
            obj.store = data['store'] if 'store' in data else None
            obj.country = data['country'] if 'country' in data else None
            obj.image_url = data['image_url'] if 'image_url' in data else None
            obj.success_url = data['success_url'] if 'success_url' in data else None

            return obj

    @staticmethod
    def provider(data=None):
        if not data:
            return Provider()
        else:
            obj = Provider()

            obj.name = data['name']
            obj.store_image = data['store_image']
            obj.is_active = data['is_active']
            obj.internal_name = data['internal_name']
            obj.image_small = data['image_small']
            obj.image_medium = data['image_medium']
            obj.image_large = data['image_large']
            obj.transaction_limit = data['transaction_limit']
            obj.rank = data['rank']

            return obj

    @staticmethod
    def sms_object(data=None):
        if not data:
            return SmsObject()
        else:
            obj = SmsObject()

            obj.id = data['id']
            obj.object = data['object']
            obj.short_id = data['short_id']

            return obj

    @staticmethod
    def sms_data(data=None):
        if not data:
            return SmsData()
        else:
            obj = SmsData()

            obj.object = Serialize.sms_object(data['object'])

            return obj

    @staticmethod
    def sms_info(data=None):
        if not data:
            return SmsInfo()
        else:
            obj = SmsInfo()

            obj.type = data['type']
            obj.object = data['object']
            obj.data = Serialize.sms_data(data['data'])

            return obj

    @staticmethod
    def webhook(data=None):
        if not data:
            return Webhook()
        else:
            obj = Webhook()

            obj.id = data['id'] if 'id' in data else None
            obj.url = data['url'] if 'url' in data else None
            obj.mode = data['mode'] if 'mode' in data else None
            obj.status = data['status'] if 'status' in data else None

            return obj



import compropago.config as config
from compropago.service import Service


class Client:
    def __init__(self, publickey, privatekey, live):
        self.publickey = publickey
        self.privatekey = privatekey
        self.live = live

        if live:
            self.deploy_uri = config.SETTINGS['api_live_uri']
        else:
            self.deploy_uri = config.SETTINGS['api_sandbox_uri']

        self.api = Service(self)

    def get_user(self):
        return self.privatekey

    def get_pass(self):
        return self.publickey

    def get_full_auth(self):
        return self.privatekey+':'+self.publickey



from compropago.tools.http import Http
import json


class Request:
    @staticmethod
    def validate_response(res):
        if res:
            aux = json.loads(res)

            if 'type' in aux and aux['type'] == 'error':
                raise Exception('Error : '+aux['message'])
            else:
                return True
        else:
            raise Exception('Empty response.')

    @staticmethod
    def get(url, auth=None, headers=None):
        http = Http(url)
        http.set_auth(auth)
        http.set_method('GET')
        http.set_extra_headers(headers)
        response = http.execute_request()

        Request.validate_response(response)

        return response

    @staticmethod
    def post(url, data=None, auth=None, headers=None):
        http = Http(url)
        http.set_method('POST')
        http.set_auth(auth)
        http.set_data(data)
        http.set_extra_headers(headers)
        response = http.execute_request()

        Request.validate_response(response)

        return response

    @staticmethod
    def put(url, data=None, auth=None, headers=None):
        http = Http(url)
        http.set_method('PUT')
        http.set_auth(auth)
        http.set_data(data),
        http.set_extra_headers(headers)
        response = http.execute_request()

        Request.validate_response(response)

        return response

    @staticmethod
    def delete(url, data=None, auth=None, headers=None):
        http = Http(url)
        http.set_method('DELETE')
        http.set_auth(auth)
        http.set_data(data)
        http.set_extra_headers(headers)
        response = http.execute_request()

        Request.validate_response(response)

        return response



from compropago.factory.factory import Factory
from compropago.factory.models.placeorderinfo import PlaceOrderInfo
from compropago.tools.request import Request


class Service:
    def __init__(self, client):
        self.client = client

    """
    # @param  [boolean] auth
    # @param  [float]   limit
    # @param  [string]  currency
    # @return [Array<Provider>]
    """
    def list_providers(self, auth=False, limit=0, currency='MXN'):
        if auth:
            uri = self.client.deploy_uri+'providers/'
            keys = {'user': self.client.get_user(), 'pass': self.client.get_pass()}
        else:
            uri = self.client.deploy_uri+'providers/true/'
            keys = None

        if limit > 0:
            uri += '?order_total='+str(limit)

        if limit > 0 and currency and currency != 'MXN':
            uri += '&currency='+currency

        response = Request.get(url=uri, auth=keys)

        return Factory.get_instance_of(class_name='ListProviders', data=response)

    """
    # @param  [string] order_id
    # @return [CpOrderInfo]
    """
    def verify_order(self, order_id):
        response = Request.get(
            self.client.deploy_uri+'charges/'+order_id+'/',
            {'user': self.client.get_user(), 'pass': self.client.get_pass()}
        )

        obj = Factory.get_instance_of('CpOrderInfo', response)

        return obj

    """
    # @param  [PlaceOrderInfo] order
    # @return [NewOrderInfo]
    """
    def place_order(self, order):
        if not isinstance(order, PlaceOrderInfo):
            raise Exception('Order object is not valid.')

        params = {
            "order_id": order.order_id,
            "order_name": order.order_name,
            "order_price": order.order_price,
            "customer_name": order.customer_name,
            "customer_email": order.customer_email,
            "payment_type": order.payment_type,
            "currency": order.currency,
            "image_url": order.image_url,
            "app_client_name": order.app_client_name,
            "app_client_version": order.app_client_version
        }

        response = Request.post(
            self.client.deploy_uri+'charges/',
            params,
            {'user': self.client.get_user(), 'pass': self.client.get_pass()}
        )

        obj = Factory.get_instance_of('NewOrderInfo', response)

        return obj

    """
    # @param  [string] number
    # @param  [string] order_id
    # @return [SmsInfo]
    """
    def send_sms_instructions(self, number, order_id):
        params = {"customer_phone": number}

        res = Request.post(
            self.client.deploy_uri+'charges/'+order_id+'/sms/',
            params,
            {'user': self.client.get_user(), 'pass': self.client.get_pass()}
        )

        obj = Factory.get_instance_of('SmsInfo', res)

        return obj

    """
    # @param  [string] url
    # @return [Webhook]
    """
    def create_webhook(self, url):
        params = {"url": url}

        res = Request.post(
            self.client.deploy_uri+'webhooks/stores/',
            params,
            {'user': self.client.get_user(), 'pass': self.client.get_pass()}
        )

        obj = Factory.get_instance_of('Webhook', res)

        return obj

    """
    # @param  [string] webhook_id
    # @param  [string] url
    # @return [Webhook]
    """
    def update_webhook(self, webhook_id, url):
        params = {"url": url}

        res = Request.put(
            self.client.deploy_uri+'webhooks/stores/'+webhook_id+'/',
            params,
            {'user': self.client.get_user(), 'pass': self.client.get_pass()}
        )

        obj = Factory.get_instance_of('Webhook', res)

        return obj

    """
    # @param  [string] webhook_id
    # @return [Webhook]
    """
    def delete_webhook(self, webhook_id):
        res = Request.delete(
            self.client.deploy_uri+'webhooks/stores/'+webhook_id+'/',
            None,
            {'user': self.client.get_user(), 'pass': self.client.get_pass()}
        )

        obj = Factory.get_instance_of('Webhook', res)

        return obj

    """
    # @return [Array<Webhook>]
    """
    def list_webhooks(self):
        res = Request.get(
            self.client.deploy_uri+'webhooks/stores/',
            {'user': self.client.get_user(), 'pass': self.client.get_pass()}
        )

        obj = Factory.get_instance_of('ListWebhooks', res)

        return obj



from compropago.tools.request import Request
from compropago.factory.factory import Factory


class Validations:
    @staticmethod
    def eval_auth(client):
        response = Request.get(
            client.deploy_uri+'users/auth',
            {'user': client.get_user(), 'pass': client.get_pass()}
        )
        info = Factory.get_instance_of('EvalAuthInfo', response)

        if info.code == 200:
            return info
        else:
            raise Exception('Error '+str(info.code)+': '+info.message)

    @staticmethod
    def validate_gateway(client):
        if not client:
            raise Exception('Client Object is not valid')

        client_mode = client.live

        auth_info = Validations.eval_auth(client)

        if auth_info.mode_key != auth_info.livemode:
            raise Exception('Keys are diferent of store mode')

        if client_mode != auth_info.livemode:
            raise Exception('Client mode is diferent of store mode')

        if client_mode != auth_info.mode_key:
            raise Exception('Client mode is diferent of keys mode')

        return True



class Provider:
    name = None
    store_image = None
    is_active = None
    internal_name = None
    image_small = None
    image_medium = None
    image_large = None
    transaction_limit = None
    rank = None

    def __init__(self):
        pass



import compropago.config as config


class PlaceOrderInfo:
    order_id = None
    order_name = None
    order_price = None
    customer_name = None
    customer_email = None
    payment_type = None
    currency = None
    image_url = None
    app_client_name = None
    app_client_version = None

    def __init__(
        self,
        order_id,
        order_name,
        order_price,
        customer_name,
        customer_email,
        payment_type="OXXO",
        currency="MXN",
        image_url="",
        app_client_name="python-sdk",
        app_client_version=config.SETTINGS['sdk_version']
    ):
        self.order_id = order_id
        self.order_name = order_name
        self.order_price = order_price
        self.customer_name = customer_name
        self.customer_email = customer_email
        self.payment_type = payment_type
        self.currency = currency
        self.image_url = image_url
        self.app_client_name = app_client_name
        self.app_client_version = app_client_version



from distutils.core import setup
setup(
  name='compropago',
  packages=[
    'compropago',
    'compropago/factory',
    'compropago/factory/models',
    'compropago/tools'
  ],
  version='1.0.0.1',
  description='SDK python para ComproPago',
  author='Eduardo Aguilar',
  author_email='eduardo.aguilar@compropago.com',
  url='https://github.com/compropago/sdk-python',
  download_url='https://github.com/compropago/sdk-python/tarball/1.0.0',
  keywords=['payments', 'sdk', 'gateway', 'compropago'],
)


import requests
import json
import base64


class Http:

    url = None
    data = None
    auth = None
    method = None
    extra_headers = None

    def __init__(self, url):
        self.url = url

    def set_method(self, method):
        if method is 'GET' or method is 'POST' or method is 'PUT' or method is 'DELETE':
            self.method = method
        else:
            raise Exception('Not supported method.')

    def set_auth(self, auth):
        if type(auth) is dict:
            aux_auth = auth['user']+':'+auth['pass']
            self.auth = base64.b64encode(bytes(aux_auth, "utf-8")).decode("utf-8")

    def set_data(self, data):
        if type(data) is dict and data:
            self.data = json.dumps(data)

    def set_extra_headers(self, headers):
        if type(headers) is dict and headers:
            self.extra_headers = headers

    def execute_request(self):
        headers = {
            'content-type': "application/json",
            'cache-control': "no-cache"
        }

        final_headers = dict()

        if self.auth:
            headers['authorization'] = self.auth

        if self.extra_headers:
            for value in zip(headers, self.extra_headers):
                final_headers[value[0]] = value[1]
        else:
            final_headers = headers

        response = requests.request(self.method, self.url, data=self.data, headers=final_headers)

        response = json.dumps(response.json())

        return response




from compropago.factory.models.feedetails import FeeDetails
from compropago.factory.models.orderinfo import OrderInfo
from compropago.factory.models.customer import Customer


class CpOrderInfo:
    id = None
    type = None
    object = None
    created = None
    paid = None
    amount = None
    livemode = None
    currency = None
    refunded = None
    fee = None
    fee_details = None
    order_info = None
    customer = None
    captured = None
    failure_message = None
    failure_code = None
    amount_refunded = None
    description = None
    dispute = None

    def __init__(self):
        self.fee_details = FeeDetails()
        self.order_info = OrderInfo()
        self.customer = Customer()



from compropago.factory.models.instructiondetails import InstructionDetails


class Instructions:
    description = None
    step_1 = None
    step_2 = None
    step_3 = None
    note_extra_comition = None
    note_expiration_date = None
    note_confirmation = None
    details = None

    def __init__(self):
        self.details = InstructionDetails()



class FeeDetails:
    amount = None
    currency = None
    type = None
    description = None
    application = None
    tax_percent = None
    amount_refunded = None
    tax = None

    def __init__(self):
        pass
